﻿#region [ COPYRIGHT ]
//::|ññññññññññññññññññññññññññññññññññññññññññññññññññññññññññññññññññññññññññññññññññññññññ|::
//::|ññ ---------------------------------------------------------------------------------- ññ|::
//::|ññ|                        ~ A R I E L   P R O D U C T I O N S ~                     |ññ|::
//::|ññ|                               ~ ALL RIGHTS RESERVED ~                            |ññ|::
//::|ññ ---------------------------------------------------------------------------------- ññ|::
//::|ññ| ~ PROGRAM DATA ~                                                       /-------/ |ññ|::
//::|ññ|                                                                      /-------/ | |ññ|::
//::|ññ| PROGRAMMER:: James Dougherty                                         | |     | | |ññ|::
//::|ññ| COPYRIGHT :: ©2009 Ariel Productions                                 | |     | | |ññ|::
//::|ññ|                                                                      | /-----|-/ |ññ|::
//::|ññ|                                                                      /-------/   |ññ|::
//::|ññ ---------------------------------------------------------------------------------- ññ|::
//::|ññ| ~ SOURCE DATA ~                                                                  |ññ|::
//::|ññ|                                                                                  |ññ|::
//::|ññ| FILENAME :: ParallaxRFID.cs                                                      |ññ|::
//::|ññ| VERSION  :: 0.1                                                                  |ññ|::
//::|ññ|                                                                                  |ññ|::
//::|ññ| HISTORY DATA                                                                     |ññ|::
//::|ññ| |                                                                                |ññ|::
//::|ññ| |-01.06.2009 JED v0.1 - Initial Build                                            |ññ|::
//::|ññ| |                                                                                |ññ|::
//::|ññ|                                                                                  |ññ|::
//::|ññ| BUGS                                                                             |ññ|::
//::|ññ| |                                                                                |ññ|::
//::|ññ| |-N/A                                                                            |ññ|::
//::|ññ| |                                                                                |ññ|::
//::|ññ|                                                                                  |ññ|::
//::|ññ ---------------------------------------------------------------------------------- ññ|::
//::|ññññññññññññññññññññññññññññññññññññññññññññññññññññññññññññññññññññññññññññññññññññññññ|::
//::|ññ ---------------------------------------------------------------------------------- ññ|::
//::|ññ| This program is Copyright ©2009 Ariel Productions (James Dougherty). This        |ññ|::
//::|ññ| program is not in the public domain, therefore, it is not open source and all	  |ññ|::
//::|ññ| copyrights remain intact. This program constitutes proprietary information and   |ññ|::
//::|ññ| trade secrets of Ariel Productions, Ashland, Ohio. This program or any part of   |ññ|::
//::|ññ| this program may not be copied, sold, rented, leased, given, assigned, or        |ññ|::
//::|ññ| otherwise made available to any other individual or organization without the     |ññ|::
//::|ññ| written consent of Ariel Productions. Also, no part of this unpublished work may |ññ|::
//::|ññ| be reproduced, transmitted, or stored in a retrieval system. Nor can this        |ññ|::
//::|ññ| program be translated into any human or computer language in any form or by any  |ññ|::
//::|ññ| means electronically, mechanically, magnetically, optically, chemically, manual  |ññ|::
//::|ññ| or otherwise without the written permission of the copyright owner               |ññ|::
//::|ññ| (Ariel Productions). Contact information is listed below for more information.   |ññ|::
//::|ññ|                                                                                  |ññ|::
//::|ññ| Ariel Productions                                                                |ññ|::
//::|ññ| James Dougherty                                                                  |ññ|::
//::|ññ| jdougherty@arielproductions.com                                                  |ññ|::
//::|ññ| http://www.arielproducitons.com/                                                 |ññ|::
//::|ññ|                                                                                  |ññ|::
//::|ññ ---------------------------------------------------------------------------------- ññ|::
//::|ññññññññññññññññññññññññññññññññññññññññññññññññññññññññññññññññññññññññññññññññññññññññ|::
#endregion
using System;
using System.Collections.Generic;
using System.Text;
using System.IO.Ports;

namespace RFID_Library
{
    class ParallaxRFID : IDisposable
    {
        #region [ Class Delegates ]
        public delegate void PinChangedDelegate();
        public delegate void PortErrorDelegate(string data);
        public delegate void DataReceivedDelegate(string data);
        public delegate void ClassWarningDelegate(string data);
        public delegate void ClassExceptionDelegate(string data);
        #endregion

        #region [ Class Events ]
        /// <summary>
        /// This event is fired if the communication port generates an error
        /// message
        /// </summary>
        public event PortErrorDelegate RFID_PortError;

        /// <summary>
        /// This event is fired on communication port pin change
        /// </summary>
        public event PinChangedDelegate RFID_PinChanged;

        /// <summary>
        /// This event is fired when the program receives data from
        /// the communication port.
        /// </summary>
        public event DataReceivedDelegate RFID_DataReceived;

        /// <summary>
        /// This event is fired if the class generates a warning message
        /// </summary>
        public event ClassWarningDelegate RFID_ClassWarning;

        /// <summary>
        /// This event is fired if the class generates an error message
        /// </summary>
        public event ClassExceptionDelegate RFID_ClassException;
        #endregion

        #region [ Class Variables ]
        /// <summary>
        /// The port that we should try and connect to (i.e. COM1. COM2, etc.)
        /// </summary>
        private string portName = "";

        /// <summary>
        /// How long should the program try to read data from the communication
        /// port before it times out? This value is in milliseconds
        /// (i.e. 1000 = 1 second, 2000 = 2 seconds, 2500 = 2.5 seconds, etc)
        /// </summary>
        private int readTimeout = 3000;

        /// <summary>
        /// The actual communication port object.
        /// </summary>
        private SerialPort communicationPort;

        /// <summary>
        /// This will let the program know whether the port is opened or if the
        /// port is closed.
        /// </summary>
        private bool portOpen = false;

        /// <summary>
        /// This will let the program know whether the communication port has been
        /// initialized and is ready to use or not.
        /// </summary>
        private bool initialized = false;
        #endregion

        #region [ Class Properties ]
        /// <summary>
        /// [READONLY] Indicates whether or not the communication port has been
        /// initialized and is ready to use or not.
        /// </summary>
        public bool Initialized
        {
            get
            {
                return initialized;
            }
        }

        /// <summary>
        /// [READONLY] Indicates whether or not the communication port is opened
        /// or closed.
        /// </summary>
        public bool PortIsOpen
        {
            get
            {
                return portOpen;
            }
        }

        /// <summary>
        /// [READONLY] Access to the communication port object
        /// </summary>
        public SerialPort CommunicationPort
        {
            get
            {
                return communicationPort;
            }
        }

        /// <summary>
        /// Get or Set the port name
        /// </summary>
        public string PortName
        {
            get
            {
                return portName;
            }
            set
            {
                portName = value;
            }
        }

        /// <summary>
        /// Get or Set the read timeout value
        /// </summary>
        public int ReadTimeout
        {
            get
            {
                return readTimeout;
            }
            set
            {
                readTimeout = value;
            }
        }

        /// <summary>
        /// Enable or disable the DTR
        /// </summary>
        public bool DTREnabled
        {
            get
            {
                if (communicationPort != null)
                {
                    return communicationPort.DtrEnable;
                }
                return false;
            }
            set
            {
                if (communicationPort != null)
                {
                    communicationPort.DtrEnable = value;
                }
            }
        }

        /// <summary>
        /// Get or set the communication ports read buffer size
        /// </summary>
        public int ReadBufferSize
        {
            get
            {
                if (communicationPort != null)
                {
                    return communicationPort.ReadBufferSize;
                }
                return 0;
            }
            set
            {
                if (communicationPort != null)
                {
                    communicationPort.ReadBufferSize = value;
                }
            }
        }

        /// <summary>
        /// Get or set the communication ports write buffer size
        /// </summary>
        public int WriteBufferSize
        {
            get
            {
                if (communicationPort != null)
                {
                    return communicationPort.WriteBufferSize;
                }
                return 0;
            }
            set
            {
                if (communicationPort != null)
                {
                    communicationPort.WriteBufferSize = value;
                }
            }
        }
        #endregion

        #region [ Class Constructor/Destructor ]
        public ParallaxRFID()
        {
            ReadTimeout = 3000;
        }

        public ParallaxRFID(int readTimeout)
        {
            ReadTimeout = readTimeout;
        }

        public ParallaxRFID(string portName)
        {
            initialized = InitializeCommunicationPort(portName, 3000);
            if (!Initialized)
            {
                OnException("The system was not initialized.");
                return;
            }
        }

        public ParallaxRFID(string portName, int readTimeout)
        {
            initialized = InitializeCommunicationPort(portName, readTimeout);
            if (!Initialized)
            {
                OnException("The system was not initialized.");
                return;
            }
        }

        ~ParallaxRFID()
        {
            this.Dispose();
        }
        #endregion

        #region [ Communication Port Functions ]
        /// <summary>
        /// This function will initialize the communication (serial) port.
        /// </summary>
        /// <param name="portName">
        /// The port that is to be initialized. Please note that this must be a valid
        /// communication (serial) port. "COM1", "COM2", etc. would be a valid ports
        /// assuming the computer has those ports available.
        /// </param>
        /// <param name="readTimeout">
        /// The timeout value to indicate how long we should give the device to respond.
        /// </param>
        /// <returns>
        /// This function will return a bool value indicating the initialize status.
        /// If the communication port has been initialized it will return true, Likewise,
        /// if the port has not been initialized the function will return false.
        /// </returns>
        public bool InitializeCommunicationPort(string portName, int readTimeout)
        {
            if (Initialized)
            {
                ReleaseCommunicationPort();
            }

            PortName = portName.Trim();
            if (PortName == null || PortName == "")
            {
                OnException("Please specify a communication port name before continuing.");
                return false;
            }

            if (!PortName.StartsWith("COM", StringComparison.OrdinalIgnoreCase))
            {
                OnException("Invalid communication port name specified.");
                return false;
            }

            if (!ValidComPort(PortName))
            {
                OnException("The communication port specified was not found.");
                return false;
            }

            communicationPort = new SerialPort(PortName, 2400);
            communicationPort.Parity = Parity.None;
            communicationPort.StopBits = StopBits.One;
            communicationPort.Handshake = Handshake.None;
            communicationPort.ReadTimeout = readTimeout;
            communicationPort.ReceivedBytesThreshold = 1;
            communicationPort.DataBits = 8;
            communicationPort.DtrEnable = true;
            communicationPort.PinChanged += new SerialPinChangedEventHandler(communicationPort_PinChanged);
            communicationPort.DataReceived += new SerialDataReceivedEventHandler(communicationPort_DataReceived);
            communicationPort.ErrorReceived += new SerialErrorReceivedEventHandler(communicationPort_ErrorReceived);

            ReadTimeout = readTimeout;
            initialized = true;
            return true;
        }

        /// <summary>
        /// This function will release the communication (serial) port.
        /// </summary>
        private void ReleaseCommunicationPort()
        {
            if (!Initialized)
            {
                return;
            }

            if (PortIsOpen)
            {
                FlushDataBuffer();
                CloseCommunicationPort();
            }

            if (communicationPort != null)
            {
                communicationPort.Dispose();
                communicationPort = null;
            }
            initialized = false;
        }

        /// <summary>
        /// This function will open the communication (serial) port.
        /// </summary>
        /// <returns>
        /// This function will return a bool value indicating the status.
        /// If the communication port has been opened it will return true, Likewise,
        /// if the port has not been opened the function will return false.
        /// </returns>
        public bool OpenCommunicationPort()
        {
            if (!Initialized)
            {
                OnException("The system is not initialized; Please initialize the system before continuing.");
                return false;
            }

            if (PortIsOpen)
            {
                OnWarning("The communication port is already open.");
                return false;
            }

            try
            {
                communicationPort.Open();
                portOpen = true;
            }
            catch (Exception e)
            {
                OnException("Unable to open the communication port " + PortName + "\r\nMessage: " + e.Message);
                return false;
            }
            return true;
        }

        /// <summary>
        /// This function will close the communication (serial) port.
        /// </summary>
        /// <returns>
        /// This function will return a bool value indicating the status.
        /// If the communication port has been closed it will return true, Likewise,
        /// if the port has not been closed the function will return false.
        /// </returns>
        public bool CloseCommunicationPort()
        {
            if (!Initialized)
            {
                OnException("The system is not initialized; Please initialize the system before continuing.");
                return false;
            }

            if (!PortIsOpen)
            {
                OnWarning("The communication port is already closed.");
                return false;
            }

            try
            {
                communicationPort.Close();
                portOpen = false;
            }
            catch (Exception e)
            {
                OnException("Unable to close the communication port " + PortName + "\r\nMessage: " + e.Message);
                return false;
            }
            return true;
        }

        /// <summary>
        /// This function will close the communication (serial) port.
        /// </summary>
        /// <returns>
        /// This function will return a bool value indicating the status.
        /// If the communication port has been closed it will return true, Likewise,
        /// if the port has not been closed the function will return false.
        /// </returns>
        private bool ValidComPort(string portName)
        {
            foreach (string port in SerialPort.GetPortNames())
            {
                if (port.ToUpper() == portName.ToUpper())
                {
                    return true;
                }
            }
            return false;
        }

        /// <summary>
        /// This function will flush the current read data buffer
        /// </summary>
        public void FlushDataBuffer()
        {
            if (!Initialized)
            {
                OnException("The system is not initialized; Please initialize the system before continuing.");
                return;
            }

            if (communicationPort != null && PortIsOpen)
            {
                communicationPort.ReadExisting();
                communicationPort.DiscardInBuffer();
            }
        }

        /// <summary>
        /// Reads a byte from the communication port
        /// </summary>
        public int ReadByte()
        {
            if (!Initialized)
            {
                OnException("The system is not initialized; Please initialize the system before continuing.");
                return 0;
            }

            if (communicationPort != null && PortIsOpen)
            {
                return communicationPort.ReadByte();
            }
            return 0;
        }

        /// <summary>
        /// Reads a char from the communication port
        /// </summary>
        public int ReadChar()
        {
            if (!Initialized)
            {
                OnException("The system is not initialized; Please initialize the system before continuing.");
                return 0;
            }

            if (communicationPort != null && PortIsOpen)
            {
                return communicationPort.ReadChar();
            }
            return 0;
        }

        /// <summary>
        /// Reads the existing buffer from the communication port
        /// </summary>
        public string ReadExisting()
        {
            if (!Initialized)
            {
                OnException("The system is not initialized; Please initialize the system before continuing.");
                return "";
            }

            if (communicationPort != null && PortIsOpen)
            {
                return communicationPort.ReadExisting();
            }
            return "";
        }

        /// <summary>
        /// Reads a line from the communication port
        /// </summary>
        public string ReadLine()
        {
            if (!Initialized)
            {
                OnException("The system is not initialized; Please initialize the system before continuing.");
                return "";
            }

            if (communicationPort != null && PortIsOpen)
            {
                return communicationPort.ReadLine();
            }
            return "";
        }

        /// <summary>
        /// Reads a string from the communication port up to the specified value
        /// </summary>
        public string ReadTo(string value)
        {
            if (!Initialized)
            {
                OnException("The system is not initialized; Please initialize the system before continuing.");
                return "";
            }

            if (communicationPort != null && PortIsOpen)
            {
                return communicationPort.ReadTo(value);
            }
            return "";
        }

        /// <summary>
        /// Write the specified data to the communication port
        /// </summary>
        public void WriteData(string data)
        {
            if (!Initialized)
            {
                OnException("The system is not initialized; Please initialize the system before continuing.");
                return;
            }

            if (communicationPort != null && PortIsOpen)
            {
                communicationPort.Write(data);
            }
        }

        /// <summary>
        /// Write the specified data to the communication port
        /// </summary>
        public void WriteData(byte[] data, int offset, int count)
        {
            if (!Initialized)
            {
                OnException("The system is not initialized; Please initialize the system before continuing.");
                return;
            }

            if (communicationPort != null && PortIsOpen)
            {
                communicationPort.Write(data, offset, count);
            }
        }

        /// <summary>
        /// Write the specified data to the communication port
        /// </summary>
        public void WriteData(char[] data, int offset, int count)
        {
            if (!Initialized)
            {
                OnException("The system is not initialized; Please initialize the system before continuing.");
                return;
            }

            if (communicationPort != null && PortIsOpen)
            {
                communicationPort.Write(data, offset, count);
            }
        }

        /// <summary>
        /// This function will handle the DataReceived event fired by the SerialPort class
        /// </summary>
        private void communicationPort_PinChanged(object sender, SerialPinChangedEventArgs e)
        {
            OnPinChanged();
        }

        /// <summary>
        /// This function will handle the DataReceived event fired by the SerialPort class
        /// </summary>
        private void communicationPort_DataReceived(object sender, SerialDataReceivedEventArgs e)
        {
            OnDataReceived(communicationPort.ReadExisting());
        }

        /// <summary>
        /// This function will handle the ErrorReceived event fired by the SerialPort class
        /// </summary>
        private void communicationPort_ErrorReceived(Object sender, SerialErrorReceivedEventArgs e)
        {
            OnPortError(communicationPort.ReadExisting());
        }
        #endregion

        #region [ Safe Event Functions ]
        /// <summary>
        /// These functions will allow us to fire the class events in a safe way.
        /// Instead of checking to see if the object has been created or not everytime
        /// we can simply call the On* functions that will do it for us. If the object
        /// has been created then the class will fire the specified event. Likewise, if
        /// the object has not been created then the event will simply be ignored.
        /// </summary>
        protected void OnPinChanged()
        {
            if (RFID_PinChanged != null)
            {
                RFID_PinChanged();
            }
        }

        protected void OnPortError(string data)
        {
            if (RFID_PortError != null)
            {
                RFID_PortError(data);
            }
        }

        protected void OnDataReceived(string data)
        {
            if (RFID_DataReceived != null)
            {
                RFID_DataReceived(data);
            }
        }

        protected void OnWarning(string data)
        {
            if (RFID_ClassWarning != null)
            {
                RFID_ClassWarning(data);
            }
        }

        protected void OnException(string data)
        {
            if (RFID_ClassException != null)
            {
                RFID_ClassException(data);
            }
        }
        #endregion

        #region [ IDisposable Functions ]
        private bool disposed;
        public void Dispose()
        {
            Dispose(true);
            GC.SuppressFinalize(this);
        }

        private void Dispose(bool bDisposing)
        {
            if (!disposed)
            {
                ReleaseCommunicationPort();
            }
            disposed = true;
        }
        #endregion
    }
}
